# !/usr/bin/env python
# faSomeRecords.py

import argparse
import textwrap
import sys


def register_arguments(parser):
    parser.add_argument('--fasta', '-f', metavar='FASTA_FILE', type=str, required=True,
                        help='FASTA file where all the sequences are stored.')
    parser.add_argument('--list', '-l', metavar='LIST', type=str, help='file name of the list.')
    parser.add_argument('--records', '-r', metavar='RECORD', nargs="*", type=str, help='individual FASTA records.')
    parser.add_argument('--outfile', '-o', default="records.fasta", nargs="?", type=str,
                        help='name for output file (default: %(default)s)')
    parser.add_argument('--stdout', '-s', action="store_true", default=False,
                        help='if sequences should be printed to screen.')
    parser.add_argument('--keep', '-k', action="store_true", default=False, help="keep the order in the list.")
    parser.add_argument('--exclude', '-e', action="store_true", default=False,
                        help="Exclude these sequences from the final output.")


def run(args):

    if args.list is None and args.records is None:
        # parser.print_usage()
        sys.exit("FaSomeRecords.py: error: argument --list/-l or --records/-r is required")
    if args.list is not None and args.records is None:
        with open(args.list, "r") as l:
            heads = l.read().splitlines()
        if ">" not in heads[0]:
            heads = [">" + h for h in heads]
    elif args.records is not None and args.list is None:
        heads = args.records
        if ">" not in heads[0]:
            heads = [">" + h for h in heads]

    requested = len(heads)
    joinheads = "|" + "|".join(heads) + "|"

    found = 0
    not_found = []

    if args.exclude:
        if args.keep:
            print("--exclude will maintain the order of the original FASTA by default")
        with open(args.outfile, "w") as out:
            with open(args.fasta, "r") as f:
                for line in f:
                    if line.startswith(">") and line.strip("\n") not in heads:
                        seq = 1
                        out.write(line)
                        found += 1
                    elif line.startswith(">"):
                        not_found += [line[1:-1]]
                        seq = 0
                    else:
                        if seq == 1:
                            out.write(line)

    elif args.keep and not args.exclude:
        store = {}
        with open(args.fasta, "r") as f:
            for line in f:
                if line[0] == ">":
                    if "|" + line[:-1] + "|" in joinheads:
                        h = line[:-1]
                        seq = 1
                        store[h] = ''
                        found += 1
                    else:
                        seq = 0
                else:
                    if seq == 1:
                        store[h] += line
        if found == 0:
            print("No sequences found")
        else:
            if args.stdout:
                for h in heads:
                    if store.get(h):
                        sys.stdout.write(h + "\n" + store[h])
                    else:
                        sys.stdout.write(h + "\n" + "#not found\n")
                        not_found += [h]
            else:
                with open(args.outfile, "w") as o:
                    for h in heads:
                        if store.get(h):
                            o.write(h + "\n" + store[h])
                        else:
                            sys.stdout.write(h + "\n" + "#not found\n")
                            not_found += [h]
                print("Found {} sequence(s)".format(found))
                if found > requested:
                    print("Found {} sequence(s) more than requested".format(found - requested))
                elif requested > found:
                    print("Could not find {} sequence(s)".format(requested - found))
                    print("\n".join(not_found))
                print("Sequences saved to: " + args.outfile)
    else:
        if args.stdout:
            with open(args.fasta, "r") as f:
                for line in f:
                    if line[0] == ">":
                        if "|" + line[:-1] + "|" in joinheads:
                            seq = 1
                            sys.stdout.write(line)
                            found += 1
                        else:
                            not_found += [line[:-1]]
                            seq = 0
                    else:
                        if seq == 1:
                            sys.stdout.write(line)
            if found == 0:
                print("No sequences found")
        else:
            with open(args.outfile, "w") as o:
                with open(args.fasta, "r") as f:
                    for line in f:
                        if line[0] == ">":
                            if "|" + line[:-1] + "|" in joinheads:
                                seq = 1
                                o.write(line)
                                found += 1
                            else:
                                not_found += [line[1:-1]]
                                seq = 0
                        else:
                            if seq == 1:
                                o.write(line)
            if found == 0:
                print("No sequences found")
            else:
                print("Found {} sequence(s)".format(found))
                if found > requested:
                    print("Found {} sequence(s) more than requested".format(found - requested))
                elif requested > found:
                    print("Could not find {} sequence(s)".format(requested - found))
                    print("\n".join(not_found))
                print("Sequences saved to " + args.outfile)


from Bio import SeqIO
import argparse


def register_arguments(parser):
    parser.add_argument('--input_fasta', '-i', type=str, help='input multi-FASTA to verify', required=True)
    parser.add_argument('--length', '-l', type=float, help='Minimum length for each record in the '
                                                           'multi-FASTA', default=29700)
    parser.add_argument('--completeness', '-c', type=float, help='Minimum genome completeness percent for each record '
                                                                 'in the multi-FASTA', default=90)


def run(args):

    fasta_sequences = SeqIO.parse(open(args.input_fasta), 'fasta')

    too_short = 0
    for record in fasta_sequences:
        if len(record) < args.length:
            too_short += 1
            print("WARNING: The following sequences are too short for analysis: Sequence: {}, Length: {}".format(record.name, len(record)))
        try:
            genome_completeness = 100 - (100 * record.seq.count("N") / len(record))
            if genome_completeness < args.completeness:
                print("WARNING: The following sequences have genome completeness below {}%: {}  {}".format(
                    args.completeness, record.name, genome_completeness))
        except ZeroDivisionError:
            print("WARNING: sequence does not have any length: {}".format(record.name) + "\n")
    if too_short == 0:
        print("All sequences are long enough for analysis")



_program = "fastafurious"
__version__ = "1.2.2"

import argparse
import re
import os
import sys
import importlib
from types import SimpleNamespace
from argparse import RawTextHelpFormatter


# make a dictionary with sub-commands and their descriptions
help_dict = {"filter": "filter sequences in FASTA based on completeness and length",
             "composition": "Print the composition statistics of FASTA sequences (completeness/length)",
             "subset": "Create a FASTA subset based on a txt list or bash record input",
             "rename": "Rename the headers of a fasta file based on the columns of a CSV file",
             "compare": "Compare the header entries of two FASTA input files"}

command_dict = {}
for command in help_dict.keys():
    command_dict[str(importlib.import_module('fastafurious.' + str(command)))] = command

# import the modules for each of the subcommands
# requires the names of the modules to be named the same as the subcommand
COMMANDS = [importlib.import_module('fastafurious.' + c) for c in help_dict.keys()]


def first_line(text):
    """
    Returns the first line of the given text, ignoring leading and trailing
    whitespace.
    """
    return text.strip().splitlines()[0]


def make_parser():
    parser = argparse.ArgumentParser(
        prog="fastafurious",
        description="fastafurious: Bundled utilities for manipulating and integrating FASTA files",
        formatter_class=RawTextHelpFormatter)

    subparsers = parser.add_subparsers(description="fastafurious provides a series of bundled functions to \n"
                                                   "easily manipulate and integrate FASTA FILES into routine \n"
                                                   "bioinformatics workflows")

    add_default_command(parser)
    # add_version_alias(parser)

    # create a unique sub-parser for each of the possible sub commands
    # requires the sub commands to have register_arguments and run commands in the script file
    for command in COMMANDS:
        # add the help description for each sub commands to its subparser
        help_line = help_dict.get(command_dict.get(str(command)))
        subparser = subparsers.add_parser(command_name(command), help=help_line,
                                              description=help_line)

        subparser.set_defaults(__command__=command)

            # Let the command register arguments on its subparser.
        command.register_arguments(subparser)

            # Use the same formatting class for every command for consistency.
            # Set here to avoid repeating it in every command's register_parser().
        subparser.formatter_class = argparse.ArgumentDefaultsHelpFormatter

    return parser


def run(argv):
    args = make_parser().parse_args(argv)
    return args.__command__.run(args)


def add_default_command(parser):
    """
    Sets the default command to run when none is provided.
    """

    class default_command():
        def run(args):
            parser.print_help()
            return 2

    parser.set_defaults(__command__ = default_command)


def command_name(command):
    """
    Returns a short name for a command module.
    """

    def remove_prefix(prefix, string):
        return re.sub('^' + re.escape(prefix), '', string)

    package = command.__package__
    module_name = command.__name__

    return remove_prefix(package, module_name).lstrip(".").replace("_", "-")





from Bio import SeqIO
from itertools import product
import pandas as pd
import argparse

# compare the headers of two FASTAs to provide content summaries

def register_arguments(parser):
    parser.add_argument('--input_fasta_1', '-1', dest="input_fasta_1", type=str,
                        help='First input FASTA to compare', required=True)
    parser.add_argument('--input-fasta_2', '-2', dest="input_fasta_2", type=str,
                        help='Second input FASTA to compare', required=True)
    parser.add_argument('--output-summary', '-o', dest="output_summary",
                        type=str, help='Optional output summary CSV file of '
                                       'header comparison', required=False)

def run(args):
    fasta_1_read = SeqIO.parse(open(args.input_fasta_1), 'fasta')
    fasta_2_read = SeqIO.parse(open(args.input_fasta_2), 'fasta')

    fasta_1_names = [record.name for record in fasta_1_read]
    fasta_2_names = [record.name for record in fasta_2_read]

    d = []

    for i, j in product(fasta_1_names, fasta_2_names):
        d.append([i, j])

    df = pd.DataFrame(d, columns=["fasta_1", "fasta_2"])

    same_entries = df[df.fasta_1 == df.fasta_2]

    # different_entries = df[df.fasta_1 != df.fasta_2]

    print("The following headers match in both input FASTAS:" + "\n")
    if same_entries.shape[0] > 0:
        print(same_entries.to_string(index=False))
        print("Number of matching FASTA sequences: {}".format(same_entries.shape[0]))
    else:
        print("No matching FASTA sequences were found between both input files")

    if args.output_summary:
        same_entries.to_csv(args.output_summary, index=False)




import pandas as pd
import os
import argparse
from Bio import SeqIO

def register_arguments(parser):
    parser.add_argument('--sample_ids', '-s', type=str, help='Sample ID txt file with current and new names',
                        required=True)
    parser.add_argument('--input_fasta', '-i', type=str, help='Input multi-fasta to convert', required=True)
    parser.add_argument('--output_fasta', '-o', type=str, help='Output file with rename headers',
                        required=True)
    parser.add_argument('--original_name', '-1', type=str, help='Column header for the existing FASTA headers',
                        required=True)
    parser.add_argument('--new_name', '-2', type=str, help='Column header for the new FASTA headers',
                        required=True)
    parser.add_argument('--keep-all', "-k", action="store_true",
                        help="If a name match is not found in the renaming CSV, retain sequence with the original name "
                             "instead of removing it.", dest = "keep_all")


def run(args):

    metadata = pd.read_csv(args.sample_ids, sep=",")
    # change the first variable to whatever the name of the column is that contain the names that you will change to
    # the index will have the names that match the current fasta names before renaming
    try:
        data_dict = pd.Series(metadata[str(args.new_name)].values, index=metadata[str(args.original_name)]).to_dict()
       
    except TypeError:
        data_dict = None
        print("The renaming dictionary could not be created:\n {}".format(data_dict))

    fasta_sequences = SeqIO.parse(open(args.input_fasta), 'fasta')

    all_in_fasta = []
    with open(args.output_fasta, "w") as handle:
        for record in fasta_sequences:
            all_in_fasta.append(record.id)
            new_name = str(data_dict.get(record.id))
            if new_name != "None":
                record.id = new_name
                record.description = record.id
                SeqIO.write(record, handle, "fasta")
            else:
                if args.keep_all:
                    record.id = record.id
                    SeqIO.write(record, handle, "fasta")
                    print("WARNING: the following record has no match in samples IDs and will be kept with the original name: {}"
                          .format(record.id))
                else:
                    pass
                    print("WARNING: --keep-all is disabled. The following record has no match in samples IDs and will be removed: {}"
                          .format(record.id))


    missing_from_fasta = [i for i in set(data_dict.keys()) if i not in all_in_fasta]
    if len(missing_from_fasta) > 0:
        print("WARNING: the following records are in the Sample IDs but missing from the input FASTA:")
        print(*missing_from_fasta, sep="\n")


from setuptools import setup
from fastafurious import __version__, _program
import os
from pathlib import Path

try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(
    name='fastafurious',
    version=__version__,
    packages=['fastafurious'],
    package_dir={'fastafurious': 'fastafurious'},
    url='https://github.com/matt-sd-watson/FASTAfurious/',
    project_urls = {
        "Issues": "https://github.com/matt-sd-watson/FASTAfurious/issues",
        "Source": "https://github.com/matt-sd-watson/FASTAfurious",
    },
    license='',
    author='Matthew Watson',
    author_email='matthew.watson@uhn.ca',
    description='Bundled utilities for manipulating and integrating FASTA files',
    long_description = long_description,
    install_requires = ["pandas>=1.1.5", "numpy>=1.19", "biopython>=1.79", "pypandoc>=1.8",
                        "pytest>=7.1.2"],
    entry_points="""
    [console_scripts]
    {program} = fastafurious.__main__:main
    """.format(program=_program),
    include_package_data=True,
)




from Bio import SeqIO
import argparse


def register_arguments(parser):
    parser.add_argument('--input_fasta', '-i', type=str, help='input multi-FASTA to verify', required=True)
    parser.add_argument('--length', '-l', type=float, help='Minimum length for each record in the '
                                                           'multi-FASTA', default=29700)
    parser.add_argument('--completeness', '-c', type=float, help='Minimum genome completeness percent for each record '
                                                                 'in the multi-FASTA', default=90)
    parser.add_argument('--output_fasta', '-o', type=str, help='output FASTA containing clean records',
                        default='cleaned.fasta')


def run(args):

    fasta_sequences = SeqIO.parse(open(args.input_fasta), 'fasta')

    count = 0
    with open(args.output_fasta, "w") as handle:
        for record in fasta_sequences:
            try:
                genome_completeness = 100 - (100 * record.seq.count("N") / len(record))
                if len(record) > args.length and genome_completeness >= args.completeness:
                    count += 1
                    SeqIO.write(record, handle, "fasta")
            except ZeroDivisionError:
                if len(record) > args.length:
                    count += 1
                    SeqIO.write(record, handle, "fasta")
                    print("WARNING: Adding record with length 0, completeness could not be calculated: {}".
                          format(record.name))

    print("{} sequences written to {}".format(count, args.output_fasta))
